#!/usr/bin/perl

#┌─────────────────────────────────
#│ WEB MART : admin.cgi - 2021/07/25
#│ copyright (c) kentweb, 1997-2021
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Net Shop v1.1 : admin.cgi - 2022/11/17
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use vars qw($cgi %in %cf);
use lib "./lib";
use CGI::Minimal;
use CGI::Session;
use Digest::SHA::PurePerl qw(sha256_base64);

# 設定ファイル取り込み
require './init.cgi';
%cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
$cgi = CGI::Minimal->new;
cgi_err('容量オーバー') if ($cgi->truncated);
%in = parse_form($cgi);

# 認証
require "./lib/login.pl";
auth_login();

# 処理分岐
if ($in{data_new}) { data_new(); }
if ($in{data_men}) { data_men(); }
if ($in{look_log}) { look_log(); }
if ($in{data_law}) { data_law(); }
if ($in{data_csv}) { data_csv(); }
if ($in{pass_mgr}) { pass_mgr(); }
if ($in{info_mgr}) { info_mgr(); }

# メニュー画面
menu_html();

#-----------------------------------------------------------
#  メニュー画面
#-----------------------------------------------------------
sub menu_html {
	header("メニューTOP");
	print <<EOM;
<div id="body">
<div class="ta-c">
<p>選択ボタンを押してください。</p>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<table class="form-tbl">
<tr>
	<th>選択</th>
	<th width="350">処理メニュー</th>
</tr><tr>
	<td class="ta-c"><input type="submit" name="data_new" value="選択"></td>
	<td class="ta-l">新規商品データ作成</td>
</tr><tr>
	<td class="ta-c"><input type="submit" name="data_men" value="選択"></td>
	<td class="ta-l">商品データメンテナンス（修正・削除）</td>
</tr><tr>
	<td class="ta-c"><input type="submit" name="info_mgr" value="選択"></td>
	<td class="ta-l">お知らせ管理</td>
</tr><tr>
	<td class="ta-c"><input type="submit" name="data_csv" value="選択"></td>
	<td class="ta-l">CSVダウン/アップロード</td>
</tr><tr>
	<td class="ta-c"><input type="submit" name="look_log" value="選択"></td>
	<td class="ta-l">ログデータ閲覧</td>
</tr><tr>
	<td><input type="submit" name="data_law" value="選択"></td>
	<td class="ta-l">特商法ページメンテナンス</td>
</tr><tr>
	<td class="ta-c"><input type="submit" name="pass_mgr" value="選択"></td>
	<td class="ta-l">パスワード管理</td>
</tr><tr>
	<td class="ta-c"><input type="button" value="選択" onclick="javascript:window.location='$cf{admin_back}'"></td>
	<td class="ta-l">商品一覧へ戻る</td>
</tr><tr>
	<td class="ta-c"><input type="submit" name="logoff" value="選択"></td>
	<td class="ta-l">ログアウト</td>
</tr>
</table>
</form>
</div>
</div>
<div id="bottom"></div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  新規商品登録
#-----------------------------------------------------------
sub data_new {
	# コード,商品名,価格,オプション
	my @log = @_;
	
	# 新規商品追加
	if ($in{job} eq "new") { data_add(); }
	
	# 在庫/軽減税率
	my %zai = read_stock()  if ($cf{stock} and @log > 0);
	my %red = read_redtax() if ($cf{tax_per} and @log > 0);
	
	# パラメータ指定
	my ($mode,$job,$input_code);
	if ($in{data_new}) {
		$mode = "data_new";
		$job = "new";
		$input_code .= qq|（英数字及びアンダーバーのみ）<br>\n|;
		$input_code .= qq|<input type="text" name="code" size="20">|;
	} else {
		$mode = "data_men";
		$job = "edit2";
		$input_code .= qq|<b>$log[0]</b> （変更不可）\n|;
		$input_code .= qq|<input type="hidden" name="code" value="$log[0]">|;
	}
	
	# 税表記
	my $tax = $cf{tax_per} == 0 ? $cf{tax_class}[0] : $cf{tax_class}[1];

	# 詳細
	my $detail;
	if ($log[0]) {
		open(IN,"$cf{datadir}/detail/$log[0].dat");
		$detail = join('',<IN>);
		close(IN);
		
		$detail =~ s/\n//g;
		$detail = encode_tag($detail);
	}

	$log[3] =~ s|\t|\n|g;
	
	header("商品登録フォーム","sn");
	back_btn();
	print <<EOM;
<div class="ttl">■ 商品登録フォーム</div>

<p>
	・ 必要項目を入力して、送信ボタンを押してください。<br>
</p>
<form action="$cf{admin_cgi}" method="post" enctype="multipart/form-data">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="$mode" value="1">
<input type="hidden" name="job" value="$job">
<table class="form-tbl03 items">
<tr>
	<th nowrap>商品コード</th>
	<td>$input_code</td>
</tr><tr>
	<th nowrap>商品名</th>
	<td><input type="text" name="item" size="40" value="$log[1]"></td>
</tr><tr>
	<th nowrap>商品単価</th>
	<td><input type="text" name="price" size="20" value="$log[2]"> 円 &nbsp; [$tax]<br>
EOM

	if ($cf{tax_per} > 0) {
		if (defined $red{$log[0]}) {
			print qq|<input type="checkbox" name="rtax" value="1" checked>\n|;
		} else {
			print qq|<input type="checkbox" name="rtax" value="1">\n|;
		}
		print qq|軽減税率対象\n|;
	}
	print "</td>\n";
	
	# 在庫管理オプション
	if ($cf{stock}) {
		print qq|</tr><tr>\n|;
		print qq|<th nowrap>在庫数</th>\n|;
		print qq|<td><input type="text" name="zai" size="4" value="$zai{$log[0]}"> 個</td>|;
	}
	
	print <<EOM;
</tr><tr>
EOM

	# 属性情報
	my $i = 8;
	for (@{$cf{options}}) {
		$i++;
		my ($key,$nam) = split(/,/);
		
		print qq|</tr><tr>\n|;
		print qq|<th nowrap>属性指定<br>[$nam]</th>\n|;
		print qq|<td>（複数をスペースで区切る。例：青 赤 黄）<br>\n|;
		print qq|<input type="text" name="op:$key" size="40" value="$log[$i]"></td>|;
	}
	
	print <<EOM;
</tr><tr>
	<th nowrap>画像</th>
	<td nowrap>
	（GIF/JPEG/PNG）<br>
EOM

	my %pic = (1 => $log[4], 2 => $log[5], 3 => $log[6], 4 => $log[7], 5 => $log[8]);

	my @head = qw|主 主 副 副 副 副|;
	foreach (1 .. 5) {
		print qq|[$head[$_]] <input type="file" name="upfile$_" size="35">\n|;
		if ($pic{$_}) {
			my ($ex,undef,undef) = split(/,/, $pic{$_});
			print qq|&nbsp;[<a href="$cf{imgurl}/$log[0]-$_$ex" target="_blank">画像</a>]\n|;
			print qq|<input type="checkbox" name="imgdel$_" value="1">削除\n|;
		}
		print "<br>\n";
	}

	print <<EOM;
	</td>
</tr><tr>
	<th nowrap>商品説明</th>
	<td>
		<textarea name="memo" class="reg-com">$log[3]</textarea>
	</td>
</tr><tr>
	<th nowrap>商品詳細内容</th>
	<td>
		（詳細画面のみに表示されます）<br>
		<textarea name="detail" id="detail">$detail</textarea>
	</td>
</tr>
</table>
<div class="submit-btn"><input type="submit" class="color prussian-blue large button" value="送信する"></div>
</form>
<hr class="bottom">
<br><br><br>
</div>
<div id="bottom"></div>
<script>
\$('#detail').summernote({
	height: 240,
	width: 800,
	lang: "ja-JP",
	fontNames: ["YuGothic","Yu Gothic","Hiragino Kaku Gothic Pro","Meiryo","sans-serif", "Arial","Arial Black","Comic Sans MS","Courier New","Helvetica Neue","Helvetica","Impact","Lucida Grande","Tahoma","Times New Roman","Verdana"],
});
</script>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  商品データ新規登録
#-----------------------------------------------------------
sub data_add {
	# 入力チェック
	check_input();
	
	# データ追加
	my ($flg,@log);
	open(DAT,"+< $cf{datadir}/shop.dat") or cgi_err("open err: shop.dat");
	eval "flock(DAT,2);";
	while(<DAT>) {
		my ($code) = (split(/<>/))[0];
		
		if ($in{code} eq $code) {
			$flg++;
			last;
		}
		push(@log,$_);
	}
	
	# コード重複のときはエラー
	if ($flg) {
		close(DAT);
		cgi_err("商品コード（$in{code}）が重複しています");
	}
	
	# オプション
	my $ops;
	foreach (@{$cf{options}}) {
		my ($key,undef) = split(/,/);
		
		$ops .= qq|$in{"op:$key"}<>|;
	}

	# 画像アップ
	my ($img1,$img2,$img3,$img4,$img5);
	if ($in{upfile1} || $in{upfile2} || $in{upfile3} || $in{upfile4} || $in{upfile5}) {
		($img1,$img2,$img3,$img4,$img5) = upload($in{code});
	}
	
	seek(DAT,0,0);
	print DAT "$in{code}<>$in{item}<>$in{price}<>$in{memo}<>$img1<>$img2<>$img3<>$img4<>$img5<>$ops\n";
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);

	# 詳細
	$in{detail} = decode_tag($in{detail});
	$in{detail} =~ s|</p>|</p>\n|ig;
	open(DAT,"> $cf{datadir}/detail/$in{code}.dat");
	print DAT $in{detail};
	close(DAT);
	
	# 軽減税率
	if ($cf{tax_per} > 0 and $in{rtax}) {
		open(DAT,">> $cf{datadir}/rtax.dat") or cgi_err("write err: rtax.dat");
		eval "flock(DAT,2);";
		print DAT "$in{code}\n";
		close(DAT);
	}
	
	# 在庫数
	if ($cf{stock}) {
		open(DAT,">> $cf{datadir}/stock.dat") or cgi_err("write err: stock.dat");
		eval "flock(DAT,2);";
		print DAT "$in{code}<>$in{zai}<>\n";
		close(DAT);
	}

	# 詳細
	$in{detail} = decode_tag($in{detail});
	$in{detail} =~ s|</p>|</p>\n|ig;
	open(DAT,"> $cf{datadir}/detail/$in{code}.dat");
	print DAT $in{detail};
	close(DAT);
	
	# 元画面フォーム
	my $btn = <<EOM;
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="data_new" value="1">
<input type="hidden" name="sid" value="$in{sid}">
<input type="submit" value="投稿フォームに戻る" style="width:140px">
</form>
EOM

	# 完了メッセージ
	message("新規商品データを追加しました", $btn);
}

#-----------------------------------------------------------
#  商品データメンテナンス
#-----------------------------------------------------------
sub data_men {
	# 指示フラグ
	my $job = $in{job};
	
	# --- 修正フォーム
	if ($job eq "edit" && $in{code}) {

		# ログ展開
		my @log;
		open(DAT,"$cf{datadir}/shop.dat") or cgi_err("open err: shop.dat");
		while(<DAT>) {
			my @data = split(/<>/);
			
			if ($in{code} == $data[0]) {
				chomp(@data);
				@log = @data;
				last;
			}
		}
		close(IN);

		# 修正フォーム
		data_new(@log);
	
	# 修正実行
	} elsif ($job eq "edit2") {
		# 入力チェック
		check_input();

		# 画像アップ
		my ($img1n,$img2n,$img3n,$img4n,$img5n);
		if ($in{upfile1} || $in{upfile2} || $in{upfile3} || $in{upfile4} || $in{upfile5}) {
			($img1n,$img2n,$img3n,$img4n,$img5n) = upload($in{code});
		}

		# 商品データオープン
		my @log;
		open(DAT,"+< $cf{datadir}/shop.dat") or cgi_err("open err: shop.dat");
		eval "flock(DAT,2);";
		while(<DAT>) {
			my @data = split(/<>/);
			
			if ($data[0] == $in{code}) {

					my ($exp,$wp,$hp) = split(/,/, $data[4]);
					my ($exp,$wp,$hp) = split(/,/, $data[5]);
					my ($exp,$wp,$hp) = split(/,/, $data[6]);
					my ($exp,$wp,$hp) = split(/,/, $data[7]);
					my ($exp,$wp,$hp) = split(/,/, $data[8]);

					my ($ex,$w,$h) = split(/,/, $img1n);
					my ($ex,$w,$h) = split(/,/, $img2n);
					my ($ex,$w,$h) = split(/,/, $img3n);
					my ($ex,$w,$h) = split(/,/, $img4n);
					my ($ex,$w,$h) = split(/,/, $img5n);

					# 削除
					if ($in{"imgdel1"} == 1) {
						unlink("$cf{imgdir}/$in{code}-1$exp");
						unlink("$cf{imgdir}/$in{code}-s-1$exp") if (-f "$cf{imgdir}/$in{code}-s-1$exp");
						$data[4] = "";
					}
					if ($in{"imgdel2"} == 1) {
						unlink("$cf{imgdir}/$in{code}-2$exp");
						unlink("$cf{imgdir}/$in{code}-s-2$exp") if (-f "$cf{imgdir}/$in{code}-s-2$exp");
						$data[5] = "";
					}
					if ($in{"imgdel3"} == 1) {
						unlink("$cf{imgdir}/$in{code}-3$exp");
						unlink("$cf{imgdir}/$in{code}-s-3$exp") if (-f "$cf{imgdir}/$in{code}-s-3$exp");
						$data[6] = "";
					}
					if ($in{"imgdel4"} == 1) {
						unlink("$cf{imgdir}/$in{code}-4$exp");
						unlink("$cf{imgdir}/$in{code}-s-4$exp") if (-f "$cf{imgdir}/$in{code}-s-4$exp");
						$data[7] = "";
					}
					if ($in{"imgdel5"} == 1) {
						unlink("$cf{imgdir}/$in{code}-5$exp");
						unlink("$cf{imgdir}/$in{code}-s-5$exp") if (-f "$cf{imgdir}/$in{code}-s-5$exp");
						$data[8] = "";
					}

					# 差替
					if ($img1n) {
						if ($ex && $ex ne $exp) {
							unlink("$cf{imgdir}/$in{code}-1$exp");
							unlink("$cf{imgdir}/$in{code}-s-1$exp") if (-f "$cf{imgdir}/$in{code}-s-1$exp");
						}
						$data[4] = "$ex,$w,$h";

						# 新添付ファイル情報
						$data[4] = $img1n;
					}
					if ($img2n) {
						if ($ex && $ex ne $exp) {
							unlink("$cf{imgdir}/$in{code}-2$exp");
							unlink("$cf{imgdir}/$in{code}-s-2$exp") if (-f "$cf{imgdir}/$in{code}-s-2$exp");
						}
						$data[5] = "$ex,$w,$h";

						# 新添付ファイル情報
						$data[5] = $img2n;
					}
					if ($img3n) {
						if ($ex && $ex ne $exp) {
							unlink("$cf{imgdir}/$in{code}-3$exp");
							unlink("$cf{imgdir}/$in{code}-s-3$exp") if (-f "$cf{imgdir}/$in{code}-s-3$exp");
						}
						$data[6] = "$ex,$w,$h";

						# 新添付ファイル情報
						$data[6] = $img3n;
					}
					if ($img4n) {
						if ($ex && $ex ne $exp) {
							unlink("$cf{imgdir}/$in{code}-4$exp");
							unlink("$cf{imgdir}/$in{code}-s-4$exp") if (-f "$cf{imgdir}/$in{code}-s-4$exp");
						}
						$data[7] = "$ex,$w,$h";

						# 新添付ファイル情報
						$data[7] = $img4n;
					}
					if ($img5n) {
						if ($ex && $ex ne $exp) {
							unlink("$cf{imgdir}/$in{code}-5$exp");
							unlink("$cf{imgdir}/$in{code}-s-5$exp") if (-f "$cf{imgdir}/$in{code}-s-5$exp");
						}
						$data[8] = "$ex,$w,$h";

						# 新添付ファイル情報
						$data[8] = $img5n;
					}

				# オプション
				my $ops;
				foreach (@{$cf{options}}) {
					my ($key,undef) = split(/,/);
					
					$ops .= qq|$in{"op:$key"}<>|;
				}

				$_ = "$data[0]<>$in{item}<>$in{price}<>$in{memo}<>$data[4]<>$data[5]<>$data[6]<>$data[7]<>$data[8]<>$ops\n";
			}
			push(@log,$_);
		}

		# 更新
		seek(DAT,0,0);
		print DAT @log;
		truncate(DAT,tell(DAT));
		close(DAT);

		# 詳細
		$in{detail} = decode_tag($in{detail});
		$in{detail} =~ s|</p>|</p>\n|ig;
		open(DAT,"> $cf{datadir}/detail/$in{code}.dat");
		print DAT $in{detail};
		close(DAT);
		
		# 軽減税率
		if ($cf{tax_per} > 0) {
			my ($flg,@log);
			open(DAT,"+< $cf{datadir}/rtax.dat") or cgi_err("open err: rtax.dat");
			eval "flock(DAT,2);";
			while(<DAT>) {
				chomp;
				
				if ($in{code} eq $_) {
					$flg++;
					next if (!$in{rtax});
				}
				push(@log,"$_\n");
			}
			if ($in{rtax} and !$flg) {
				push(@log,"$in{code}\n");
			}
			seek(DAT,0,0);
			print DAT @log;
			truncate(DAT,tell(DAT));
			close(DAT);
		}
		
		# 在庫更新
		if ($cf{stock}) {
			my ($flg,@log);
			open(DAT,"+< $cf{datadir}/stock.dat") or cgi_err("open err: stock.dat");
			eval "flock(DAT,2);";
			while(<DAT>) {
				my ($code,$zai) = split(/<>/);
				
				if ($in{code} eq $code) {
					$flg++;
					$_ = "$code<>$in{zai}<>\n";
				}
				push(@log,$_);
			}
			
			# 新規追加のとき
			if (!$flg) { push(@log,"$in{code}<>$in{zai}<>\n"); }
			
			seek(DAT,0,0);
			print DAT @log;
			truncate(DAT,tell(DAT));
			close(DAT);
		}
		# 完了メッセージ
		message("商品を修正しました");
	
	# --- 削除
	} elsif ($job eq "dele" && $in{code}) {

		my @log;
		open(DAT,"+< $cf{datadir}/shop.dat") or cgi_err("open err: shop.dat");
		eval "flock(DAT, 2);";
		while(<DAT>) {
		chomp;
		my ($code,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,@ops) = split(/<>/);

			next if ($in{code} eq $code);

			if ($img1) {
				my ($ex,undef,undef) = split(/,/, $img1);
				unlink("$cf{imgdir}/$code-1$ex");
				unlink("$cf{imgdir}/$code-s-1$ex");
			}
			if ($img2) {
				my ($ex,undef,undef) = split(/,/, $img2);
				unlink("$cf{imgdir}/$code-2$ex");
				unlink("$cf{imgdir}/$code-s-2$ex");
			}
			if ($img3) {
				my ($ex,undef,undef) = split(/,/, $img3);
				unlink("$cf{imgdir}/$code-3$ex");
				unlink("$cf{imgdir}/$code-s-3$ex");
			}
			if ($img4) {
				my ($ex,undef,undef) = split(/,/, $img4);
				unlink("$cf{imgdir}/$code-4$ex");
				unlink("$cf{imgdir}/$code-s-4$ex");
			}
			if ($img5) {
				my ($ex,undef,undef) = split(/,/, $img5);
				unlink("$cf{imgdir}/$code-5$ex");
				unlink("$cf{imgdir}/$code-s-5$ex");
			}

			push(@log,$_);
		}
		seek(DAT,0,0);
		print DAT @log;
		truncate(DAT,tell(DAT));
		close(DAT);
		
		# 軽減税率
		if ($cf{tax_per} > 0) {
			my @log;
			open(DAT,"+< $cf{datadir}/rtax.dat") or cgi_err("open err: rtax.dat");
			eval "flock(DAT,2);";
			while(<DAT>) {
				chomp;
				
				next if ($in{code} eq $_);
				push(@log,"$_\n");
			}
			seek(DAT,0,0);
			print DAT @log;
			truncate(DAT,tell(DAT));
			close(DAT);
		}
		
		# 在庫更新
		if ($cf{stock}) {
			my @log;
			open(DAT,"+< $cf{datadir}/stock.dat") or cgi_err("open err: stock.dat");
			eval "flock(DAT,2);";
			while(<DAT>) {
				my ($code,$zai) = split(/<>/);
				
				next if ($in{code} eq $code);
				push(@log,$_);
			}
			seek(DAT,0,0);
			print DAT @log;
			truncate(DAT,tell(DAT));
			close(DAT);
		}

		# 詳細
		unlink("$cf{datadir}/detail/$in{code}.dat");
	}

	# 在庫データ認識
	my %zai = read_stock() if ($cf{stock});
	
	header("メニューTOP ＞ 商品データメンテナンス");
	back_btn();
	print <<EOM;
<div class="ttl">■ 商品データメンテナンス</div>

<p>処理を選択し、送信ボタンを押してください。</p>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="data_men" value="1">
処理：
<select name="job">
<option value="edit">修正
<option value="dele">削除
</select>
<input type="submit" value="送信">
<table class="form-tbl02">
<tr>
	<th nowrap>選択</th>
	<th nowrap>商品コード</th>
	<th nowrap>商品名</th>
	<th nowrap>金額</th>
EOM

	print qq|<th nowrap>在庫</th>| if ($cf{stock});
	print qq|</tr>\n|;
	
	open(IN,"$cf{datadir}/shop.dat") or cgi_err("open err: shop.dat");
	while(<IN>) {
		my ($code,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,$color,$size) = split(/<>/);
		$price = comma($price);
		
		print qq|<tr><td class="ta-c"><input type="radio" name="code" value="$code"></td>|;
		print qq|<td nowrap>$code</td>|;
		print qq|<td nowrap>$item</td>|;
		print qq|<td nowrap class="ta-r">￥$price</td>|;
		print qq|<td nowrap class="ta-r">$zai{$code}<br></td>| if ($cf{stock});
		print qq|</tr>\n|;
	}
	close(IN);
	
	print <<EOM;
</table>
</form>
</div>
<div id="bottom"></div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  お知らせ管理
#-----------------------------------------------------------
sub info_mgr {
	# 対象ファイル判定
	$cf{info} = "$cf{datadir}/news.dat";

	# --- 編集実行
	if ($in{edit}) {
		# 改行
		$in{body} =~ s/\r\n/\n/g;
		$in{body} =~ s/\r/\n/g;
		$in{body} =~ s/\t/\n/g;

		# タグ復元
		$in{body} = decode_tag($in{body});

		# 更新
		open(DAT,"> $cf{info}") or error("write err: $cf{info}");
		print DAT $in{body};
		close(DAT);

		# 完了画面
		message('編集を完了しました');
	}

	# 対象ファイル読み込み
	open(IN,"$cf{info}") or error("open err: $cf{info}");
	my $info = join('',<IN>);
	close(IN);

	# 画面表示
	header("お知らせ管理","sn");
	back_btn();
	print <<EOM;
<div id="main-in">
<p class="ttl"><img src="$cf{cmnurl}/information.png" class="icon"> お知らせ管理</p>
<p>・ お知らせの編集を行います。</p>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="info_mgr" value="1">
<table class="list">
<tr>
	<th class="ta-c">内容</th>
	<td>
		<textarea name="body" class="reg-com2" id="detail">$info</textarea>
	</td>
</tr>
</table>
<input type="submit" name="edit" value="編集する" class="bigbtn">
</form>
</div>
<script>
\$('#detail').summernote({
	height: 240,
	width: 800,
	lang: "ja-JP",
	fontNames: ["YuGothic","Yu Gothic","Hiragino Kaku Gothic Pro","Meiryo","sans-serif", "Arial","Arial Black","Comic Sans MS","Courier New","Helvetica Neue","Helvetica","Impact","Lucida Grande","Tahoma","Times New Roman","Verdana"],
});
</script>
</div>
<div id="bottom"></div>
EOM
	footer();
}

#-----------------------------------------------------------
#  ページボタン作成
#-----------------------------------------------------------
sub make_btn {
	my ($i,$pg) = @_;
	
	# ページ繰越数
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};
	
	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<span class="pg-on">$x</span>!);
			} else {
				push(@pg,qq!<span class="pg-off"><a href="$cf{find_cgi}?pg=$y">$x</a></span>!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<span class="pg-off"><a href="$cf{find_cgi}?pg=$back">&lt;</a></span>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<span class="pg-off"><a href="$cf{find_cgi}?pg=$next">&gt;</a></span>\n!;
	}
	$ret;
}

#-----------------------------------------------------------
#  アップロード
#-----------------------------------------------------------
sub upload {
	my $num = shift;

	# サムネイル機能
	require './lib/thumb.pl' if ($cf{thumbnail});

	my @ret;
	foreach my $i (1 .. 5) {

		# mimeタイプ
		my $mime = $cgi->param_mime("upfile$i");

		# ファイル名
		my $fname = $cgi->param_filename("upfile$i");
		if ($fname =~ /(\.jpe?g|\.png|\.gif)$/i) {
			my $ex = $1;
			if ($ex eq '.jpeg') { $ex = '.jpg'; }
			$ex =~ tr/A-Z/a-z/;

			# 整合性チェック
			my $flg;
			if ($cf{ok_gif}) {
				if ($mime =~ /^image\/gif$/i and $ex eq '.gif') { $flg++; }
			}
			if (!$flg and $cf{ok_jpeg}) {
				if ($mime =~ /^image\/p?jpe?g$/i and $ex eq '.jpg') { $flg++; }
			}
			if (!$flg and $cf{ok_png}) {
				if ($mime =~ /^image\/(x-)?png$/i and $ex eq '.png') { $flg++; }
			}
			if (!$flg) { error('このファイルは取り扱いできません'); }

			# アップファイル定義
			my $upfile = "$cf{imgdir}/$num-$i$ex";

			# 書き込み
			open(OUT,"+> $upfile") or error("画像アップ失敗");
			binmode(OUT);
			print OUT $in{"upfile$i"};
			close(OUT);

			# パーミッション付与
			chmod(0666,$upfile);

			# 画像サイズ取得
			my ($w,$h);
			if ($ex eq ".jpg") { ($w,$h) = j_size($upfile); $flg++; }
			elsif ($ex eq ".gif") { ($w,$h) = g_size($upfile); $flg++; }
			elsif ($ex eq ".png") { ($w,$h) = p_size($upfile); $flg++; }

			push(@ret,"$ex,$w,$h");
		}
	}
	# 返り値
	return @ret;
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;

	my ($h, $w, $t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);

		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);

	return ($w, $h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;

	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);

	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));

	return ($w, $h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;

	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);

	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));

	return ($w, $h);
}

#-----------------------------------------------------------
#  日時換算
#-----------------------------------------------------------
sub chg_date {
	my $time = shift;
	
	my ($mday,$mon,$year) = (localtime($time))[3..6];
	sprintf("%04d/%02d/%02d",$year+1900,$mon+1,$mday);
}

#-----------------------------------------------------------
#  ログデータ閲覧
#-----------------------------------------------------------
sub look_log {
	# 閲覧ボタン確認
	my $look;
	for ( keys %in ) {
		if (/^look:(\d+)$/) {
			$look = $1;
			last;
		}
	}
	# 個別閲覧
	if ($look) { look_num($look); }
	
	# 年月データ
	my $q_ym = $in{ym};
	
	# 入力内容が規定外の場合は現在の年月
	if ($q_ym !~ /^\d{6}$/) {
		my ($m,$y) = (localtime())[4,5];
		$q_ym = sprintf("%04d%02d", $y+1900,$m+1);
	}
	
	my @dir;
	opendir(DIR,"$cf{datadir}/log") or cgi_err("open err: $cf{datadir}/log");
	while( my $dir = readdir(DIR) ) {
		if ($dir =~ /^(\d+)\.cgi$/) {
			push(@dir,$1);
		}
	}
	closedir(DIR);
	
	header("メニューTOP ＞ ログデータ閲覧");
	back_btn();
	print <<EOM;
<div class="ttl">■ ログデータ閲覧</div>
<p>年月を切り替えて閲覧することができます。</p>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="look_log" value="1">
年月：
<select name="ym">
EOM

	my ($i,$top);
	for ( sort{ $b <=> $a } @dir ) {
		if (/^(\d{4})(\d{2})/) {
			if ($q_ym == "$1$2") {
				print qq|<option value="$1$2" selected>$1年$2月\n|;
			} else {
				print qq|<option value="$1$2">$1年$2月\n|;
			}
		}
		$i++;
		if ($i == 1) { $top = $_; }
	}
	
	print <<EOM;
</select>
<input type="submit" value="切替">
<table class="form-tbl">
<tr>
	<th nowrap>選択</th>
	<th nowrap>注文日時</th>
	<th nowrap>注文番号</th>
</tr>
EOM

	open(IN,"$cf{datadir}/log/$q_ym.cgi");
	while(<IN>) {
		my ($date,$num,$log) = split(/<>/);
		
		print qq|<tr><td><input type="submit" name="look:$num" value="閲覧"></td>\n|;
		print qq|<td nowrap>$date</td>\n|;
		print qq|<td nowrap class="ta-c">$num</td></tr>\n|;
	}
	close(IN);
	
	print <<EOM;
</table>
</form>
</div>
<div id="bottom"></div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  個別ログ閲覧
#-----------------------------------------------------------
sub look_num {
	my $look = shift;
	
	# ログデータ名
	my $log = ($in{ym} =~ /^(\d{4})(\d{2})$/) && "$1$2.cgi";
	
	# ログオープン
	my $data;
	open(IN,"$cf{datadir}/log/$log") or cgi_err("open err: $log");
	while(<IN>) {
		my ($date,$num,$body) = split(/<>/);
		
		if ($look == $num) {
			$data = $_;
			last;
		}
	}
	close(IN);
	
	my ($date,$num,$body) = split(/<>/,$data);
	$body =~ s/\t/<br>/g;
	
	header("メニューTOP ＞ ログデータ閲覧 ＞ 個別ログ");
	back_btn('look_log');
	print <<EOM;
<div class="ttl">■ ログデータ閲覧 ＞ 個別ログ</div>
<p>
	・注文年月： <b>$date</b><br>
	・注文番号： <b>$num</b><br>
</p>
$body
</div>
<div id="bottom"></div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  特商法メンテ
#-----------------------------------------------------------
sub data_law {
	# 新規
	if ($in{add} && $in{key} && $in{val}) {
		$in{key} =~ s/\t//g;
		$in{val} =~ s/\t//g;
		
		my $i = 0;
		open(DAT,"+< $cf{datadir}/law.dat") or cgi_err("open err: law.dat");
		while(<DAT>) {
			my ($no,$key,$val) = split(/\t/);
			if ($i < $no) { $i = $no; }
		}
		$i++;
		seek(DAT,0,2);
		print DAT "$i\t$in{key}\t$in{val}\n";
		truncate(DAT,tell(DAT));
		close(DAT);
	
	# ソート
	} elsif ($in{sort}) {
		my (@tmp,@log);
		open(DAT,"+< $cf{datadir}/law.dat") or cgi_err("open err: law.dat");
		while(<DAT>) {
			my ($no,$key,$val) = split(/\t/);
			
			push(@tmp,$in{"sort:$no"});
			push(@log,$_);
		}
		@log = @log[sort{$tmp[$a] <=> $tmp[$b]} 0..$#tmp];
		seek(DAT,0,0);
		print DAT @log;
		truncate(DAT,tell(DAT));
		close(DAT);
	}
	
	my ($chg,$del);
	for ( keys %in ) {
		if (/^chg:(\d+)/) {
			$chg = $1;
			last;
		} elsif (/^del:(\d+)/) {
			$del = $1;
			last;
		}
	}
	# 修正
	if ($chg) {
		my @log;
		open(DAT,"+< $cf{datadir}/law.dat") or cgi_err("open err: law.dat");
		while(<DAT>) {
			my ($no,$key,$val) = split(/\t/);
			
			if ($chg == $no) {
				$in{"key:$no"} =~ s/\t//g;
				$in{"val:$no"} =~ s/\t//g;
				$_ = qq|$no\t$in{"key:$no"}\t$in{"val:$no"}\n|;
			}
			push(@log,$_);
		}
		seek(DAT,0,0);
		print DAT @log;
		truncate(DAT,tell(DAT));
		close(DAT);
		
		message("修正を完了しました");
	
	# 削除
	} elsif ($del) {
		my @log;
		open(DAT,"+< $cf{datadir}/law.dat") or cgi_err("open err: law.dat");
		while(<DAT>) {
			my ($no,$key,$val) = split(/\t/);
			next if ($del == $no);
			
			push(@log,$_);
		}
		seek(DAT,0,0);
		print DAT @log;
		truncate(DAT,tell(DAT));
		close(DAT);
	}
	
	header("メニューTOP ＞ 特商法メンテナンス");
	back_btn();
	print <<EOM;
<div class="ttl">■ 特商法メンテナンス</div>
<p>改行は無効です。</p>
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="data_law" value="1">
<table class="form-tbl">
<tr>
	<th><input type="submit" name="sort" value="並替"></th>
	<th>項目</th>
	<th>内容</th>
	<th>操作</th>
</tr><tr>
	<td></td>
	<td><textarea name="key" cols="20" rows="3"></textarea></td>
	<td><textarea name="val" cols="50" rows="3"></textarea></td>
	<td><input type="submit" name="add" value="追加"></td>
</tr>
EOM

	# データ読み取り
	my $i;
	open(IN,"$cf{datadir}/law.dat");
	while(<IN>) {
		chomp;
		my ($no,$key,$val) = split(/\t/);
		
		$i++;
		print qq|<tr><td><input type="text" name="sort:$no" value="$i" size="2"></td>\n|;
		print qq|<td><textarea name="key:$no" cols="20" rows="3">$key</textarea></td>\n|;
		print qq|<td><textarea name="val:$no" cols="50" rows="3">$val</textarea></td>\n|;
		print qq|<td><input type="submit" name="chg:$no" value="修正">\n|;
		print qq|<input type="submit" name="del:$no" value="削除" onclick="return confirm('削除しますか？');"></td></tr>\n|;
	}
	close(IN);
	
	print <<EOM;
</table>
</form>
</div>
<div id="bottom"></div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  CSVダウン/アップロード
#-----------------------------------------------------------
sub data_csv {
	# ダウンロード
	if ($in{downld}) {
		require "./lib/jacode.pl";
		
		open(DAT,"$cf{datadir}/shop.dat") or error("open err: shop.dat");
		
		# ダウンロード用ヘッダ
		print "Content-type: application/octet-stream\n";
		print "Content-Disposition: attachment; filename=data.csv\n\n";
		
		# バイナリ出力 (Windowsサーバ環境)
		binmode(STDOUT);
		
		while ( my $log = <DAT> ) {
			chomp($log);
			jcode::convert(\$log,'sjis','utf8');
			
			my $csv;
			foreach ( split(/<>/,$log) ) {
				$csv .= "$_,";
			}
			print "$csv\r\n";
		}
		close(DAT);
		exit;
	
	# ダウンロード
	} elsif ($cf{stock} && $in{downld2}) {
		
		open(DAT,"$cf{datadir}/stock.dat") or error("open err: stock.dat");
		
		# ダウンロード用ヘッダ
		print "Content-type: application/octet-stream\n";
		print "Content-Disposition: attachment; filename=stock.csv\n\n";
		
		# バイナリ出力 (Windowsサーバ環境)
		binmode(STDOUT);
		
		while (<DAT>) {
			my ($no,$zai) = split(/<>/);
			print "$no,$zai\r\n";
		}
		close(DAT);
		exit;
	
	# ダウンロード
	} elsif ($cf{tax_per} > 0 && $in{downld3}) {
		
		open(DAT,"$cf{datadir}/rtax.dat") or error("open err: rtax.dat");
		
		# ダウンロード用ヘッダ
		print "Content-type: application/octet-stream\n";
		print "Content-Disposition: attachment; filename=rtax.csv\n\n";
		
		# バイナリ出力 (Windowsサーバ環境)
		binmode(STDOUT);
		
		while (<DAT>) {
			chomp;
			print "$_\r\n";
		}
		close(DAT);
		exit;
	
	# アップロード
	} elsif ($in{upload} && $in{upfile}) {
		require "./lib/jacode.pl";
		
		# ファイル名
		my $fname = $cgi->param_filename("upfile");
		if ($fname !~ /\.csv$/i) { error('CSVファイルのみアップロードできます。'); }
		
		my @log;
		foreach my $log ( split(/\r\n|\r|\n/,$in{upfile}) ) {
			jcode::convert(\$log,'utf8','sjis');
			
			my $csv;
			foreach ( split(/,/,$log) ) {
				$csv .= "$_<>";
			}
			push(@log,"$csv\n");
		}
		
		# 上書き
		open(DAT,"> $cf{datadir}/shop.dat") or error("write err: shop.dat");
		print DAT @log;
		close(DAT);
		
		message('CSVをアップ更新しました');
	
	# アップロード
	} elsif ($cf{stock} && $in{upload2} && $in{upfile2}) {
		
		# ファイル名
		my $fname = $cgi->param_filename("upfile2");
		if ($fname !~ /\.csv$/i) { error('CSVファイルのみアップロードできます。'); }
		
		my $i;
		my @log;
		for ( split(/\r\n|\r|\n/,$cgi->param('upfile2')) ) {
			$i++;
			my ($no,$zai) = split(/,/);
			push(@log,"$no<>$zai<>\n");
		}
		
		# 上書き
		open(DAT,"> $cf{datadir}/stock.dat") or error("write err: stock.dat");
		print DAT @log;
		close(DAT);
		
		message("在庫データを$i個アップ更新しました");
	
	# アップロード
	} elsif ($cf{tax_per} > 0 && $in{upload3} && $in{upfile3}) {
		
		# ファイル名
		my $fname = $cgi->param_filename("upfile3");
		if ($fname !~ /\.csv$/i) { error('CSVファイルのみアップロードできます。'); }
		
		my $i;
		my @log;
		for ( split(/\r\n|\r|\n/,$cgi->param('upfile3')) ) {
			$i++;
			push(@log,"$_\n");
		}
		
		# 上書き
		open(DAT,"> $cf{datadir}/rtax.dat") or error("write err: rtax.dat");
		print DAT @log;
		close(DAT);
		
		message("軽減税率データを$i個アップ更新しました");
	}
	
	header("メニューTOP ＞ CSVダウン/アップロード");
	back_btn();
	print <<EOM;
<div class="ttl">■ CSVダウン/アップロード</div>
<p>データをCSV形式でダウンロードすることができます。</p>
<form action="$cf{admin_cgi}" method="post" enctype="multipart/form-data">
<input type="hidden" name="sid" value="$in{sid}">
<input type="hidden" name="data_csv" value="1">
<table class="form-tbl">
<tr>
	<th>CSVダウンロード</th>
	<td>
		<p>商品ファイル <input type="submit" name="downld" value="ダウンロード"></p>
EOM

	if ($cf{stock}) {
		print qq|<p>在庫ファイル <input type="submit" name="downld2" value="ダウンロード"></p>\n|;
	}
	if ($cf{tax_per} > 0) {
		print qq|<p>軽減税率ファイル <input type="submit" name="downld3" value="ダウンロード"></p>\n|;
	}
	
	print <<EOM;
	</td>
</tr>
</table>
<p>データをCSV形式でアップロードすることができます。</p>
<table class="form-tbl">
<tr>
	<th>CSVアップロード</th>
	<td>
		<p>商品ファイル <input type="file" name="upfile" size="32">
			<input type="submit" name="upload" value="アップロード"></p>
EOM

	if ($cf{stock}) {
		print qq|<p>在庫ファイル <input type="file" name="upfile2" size="25">\n|;
		print qq|<input type="submit" name="upload2" value="アップロード"></p>\n|;
	}
	if ($cf{tax_per} > 0) {
		print qq|<p>軽減税率ファイル <input type="file" name="upfile3" size="25">\n|;
		print qq|<input type="submit" name="upload3" value="アップロード"></p>\n|;
	}
	
	print <<EOM;
	</td>
</tr>
</table>
<p>
	【参考】 商品データCSVフォーマット<br>
	<span style="letter-spacing:1px;color:red">商品コード,商品名,単価,備考,戻り先,属性情報1,属性情報2, ... [改行]</span><br>
	↑上記で、属性情報とは「サイズ,カラー」等で、init.cgiで指定する\$cf{options}の個数分が続く。<br>
	（例）属性情報が「サイズ,カラー」であれば、<span style="letter-spacing:1px;color:red">商品コード,商品名,単価,備考,戻り先,サイズ,カラー [改行]</span>
</p>
EOM

	if ($cf{stock}) {
		print qq|<p>【参考】 在庫データCSVフォーマット<br>\n|;
		print qq|<span style="letter-spacing:1px;color:red">商品コード,在庫数 [改行]</span></p>\n|;
	}
	if ($cf{tax_per} > 0) {
		print qq|<p>【参考】 軽減税率CSVフォーマット<br>\n|;
		print qq|<span style="letter-spacing:1px;color:red">商品コード [改行]</span></p>\n|;
	}
	
	print <<EOM;
</form>
</div>
<div id="bottom"></div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  入力チェック
#-----------------------------------------------------------
sub check_input {
	# 禁止文字排除
	$in{code}  =~ s/\W//g;
	$in{price} =~ s/\D//g;
	
	# 在庫のとき
	if ($cf{stock}) {
		$in{zai} =~ s/\D//g;
		if ($in{zai} eq "") { $in{zai} = 0; }
	}
	
	# 入力チェック
	my $err;
	if ($in{code} eq "") { $err .= "商品コードが未入力です<br>"; }
	if ($in{item} eq "") { $err .= "商品名が未入力です<br>"; }
	if ($in{price} eq "") { $err .= "商品単価が未入力です<br>"; }
	if ($err) { cgi_err($err); }
}

#-----------------------------------------------------------
#  HTMLヘッダー
#-----------------------------------------------------------
sub header {
	my ($ttl,$sn) = @_;
	
	print <<EOM;
Content-type: text/html; charset=utf-8

<!doctype html>
<html lang="ja">
<head>
<meta charset="UTF-8">
<link href="$cf{cmnurl}/admin.css" rel="stylesheet">
<script type="text/javascript" src="./cmn/js/footerFixed_modified_admin.js"></script>
<title>$ttl</title>
EOM

	if ($sn eq 'sn') {
		print qq|<script src="$cf{cmnurl}/jquery/jquery.min.js"></script>\n|;
		print qq|<link href="$cf{cmnurl}/summernote/summernote-lite.min.css" rel="stylesheet">\n|;
		print qq|<script src="$cf{cmnurl}/summernote/summernote-lite.min.js"></script>\n|;
		print qq|<script src="$cf{cmnurl}/summernote/lang/summernote-ja-JP.js"></script>\n|;
	}
	
	print <<EOM;
</head>
<body>
<div id="head02">
	<div id="main-ttl">
		<h1>$cf{cart_title} : 管理モード</h1>
	</div>
</div>
EOM
}

#-----------------------------------------------------------
#  戻りボタン
#-----------------------------------------------------------
sub back_btn {
	my $mode = shift;
	
	print <<EOM;
<div id="body">
<div class="back-btn">
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
@{[ $mode ? qq|<input type="submit" name="$mode" value="&lt; 前画面">| : "" ]}
<input type="submit" value="&lt; メニュー">
</form>
</div>
EOM
}

#-----------------------------------------------------------
#  エラー
#-----------------------------------------------------------
sub cgi_err {
	my $msg = shift;
	
	header("ERROR!");
	print <<EOM;
<div id="body">
<div class="ta-c">
<hr width="350">
<h3>ERROR!</h3>
<p style="color:#dd0000">$msg</p>
<hr width="350">
<input type="button" value="前画面に戻る" onclick="history.back()">
</div>
</div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  メッセージ表示
#-----------------------------------------------------------
sub message {
	my ($msg,$btn) = @_;
	
	header("処理完了");
	print <<EOM;
<div id="body">
<div class="ttl">■ 処理完了</div>
<div class="msg">
	$msg
	$btn
<form action="$cf{admin_cgi}" method="post">
<input type="hidden" name="sid" value="$in{sid}">
<br><input type="submit" value="メニューに戻る" style="width:140px">
</form>
</div>
</div>
</body>
</html>
EOM
	footer();
	exit;
}

#-----------------------------------------------------------
#  タグ復元
#-----------------------------------------------------------
sub decode_tag {
	local($_) = @_;
	
	s/&gt;/>/g;
	s/&lt;/</g;
	s/&quot;/"/g;
	s/&amp;/&/g;
	s/&#39;/'/g;
	
	return $_;
}


#-----------------------------------------------------------
#  タグ変換
#-----------------------------------------------------------
sub encode_tag {
	local($_) = @_;
	
	s/&/&amp;/g;
	s/>/&gt;/g;
	s/</&lt;/g;
	s/"/&quot;/g;
	s/'/&#39;/g;
	
	return $_;
}

#-----------------------------------------------------------
#  在庫数
#-----------------------------------------------------------
sub read_stock {
	my %ret;
	open(IN,"$cf{datadir}/stock.dat");
	while(<IN>) {
		my ($code,$file) = split(/<>/);
		
		$ret{$code} = $file;
	}
	close(IN);
	
	return %ret;
}


