#!/usr/bin/perl

#┌─────────────────────────────────
#│ WEB MART : find.cgi - 2021/04/25
#│ copyright (c) kentweb, 1997-2021
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  Net Shop v1.1 : find.cgi - 2022/11/17
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 外部ファイル取り込み
require './init.cgi';
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# 処理分岐
find_cart();

#-----------------------------------------------------------
#  商品検索
#-----------------------------------------------------------
sub find_cart {
	# 引数
	$in{pg} =~ s/\D//g;
	$in{sort} =~ s/[^1-4]//g;
	
	# 軽減税率
	my %red = read_redtax() if ($cf{tax_per} > 0);
	
	# キーワードを配列化
	$in{word} =~ s/　/ /g;
	my @wd = split(/\s+/,$in{word});
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';
	
	# データファイルオープン
	my (@log,@tmp);
	open(IN,"$cf{datadir}/shop.dat") or error("open err: shop.dat");
	while(<IN>) {
		my ($item,$price,$memo) = (split(/<>/))[1..3];
		
		# キーワード
		if ($in{word}) {
			my $flg;
			foreach my $wd (@wd) {
				if ("$item $memo" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
					$flg++;
				} else {
					$flg = 0;
					last;
				}
			}
			next if (!$flg);
		}
		
		# ソート振り分け
		if ($in{sort} <= 2) {
			push(@tmp,$price);
		} elsif ($in{sort} == 3) {
			unshift(@log,$_);
			next;
		}
		push(@log,$_);
	}
	close(IN);
	
	# 高価順
	if ($in{sort} == 2) {
		@log = @log[sort {$tmp[$b] <=> $tmp[$a]} 0 .. $#tmp];
	# 安価順
	} elsif ($in{sort} == 1) {
		@log = @log[sort {$tmp[$a] <=> $tmp[$b]} 0 .. $#tmp];
	}
	
	# ヒット数
	my $hits = @log;
	
	# ページ数定義
	my $pg = $in{pg} || 0;
	
	# ページボタン作成
	my $pbtn = make_btn($hits,$pg);
	
	# 表示範囲
	my $begin = $pg;
	my $end   = $pg + $cf{pg_max} - 1;
	if ($end > $hits - 1) { $end = $hits - 1; }
	
	# オプション
	my %sort = (1 => '価格の安い順', 2 => '価格の高い順', 3 => '登録の新着順', 4 => '登録の古い順');
	my $op_sort;
	foreach (1 .. 4) {
		if ($in{sort} == $_) {
			$op_sort .= qq|<option value="$_" selected>$sort{$_}</option>\n|;
		} else {
			$op_sort .= qq|<option value="$_">$sort{$_}</option>\n|;
		}
	}

	# サイドメニュー内お知らせ
	my $news = read_news();
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/find.html") or error('open err: find.html');
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 置き換え
	$tmpl =~ s/!(\w+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!cart_title!/$cf{cart_title}/g;
	$tmpl =~ s/!top_back!/$cf{top_back}/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s/<!-- news -->/$news/;
	$tmpl =~ s/!word!/$in{word}/g;
	$tmpl =~ s/!hits!/$hits/g;
	$tmpl =~ s/<!-- op_sort -->/$op_sort/g;
	$tmpl =~ s|<!-- page_btn -->|<p>$pbtn</p>|g;
	$tmpl =~ s/!back_list!/$cf{top_cgi}/g;

	# 買物情報取得
	my ($cart,undef) = get_cookie_cart();

	# 商品データ認識
	my %cart = get_data();

	# 商品内容を展開
	my $cart_now;
	my $all = 0;
	for (@{$cart}) {
		my ($id,$code,$num,@op) = split(/,/);
			next if (!$cart);
		my (undef,$name,$price,undef,undef) = split(/<>/,$cart{$code});
		
		# チェック
		$id =~ s/\D//g;
		$code =~ s/\W//g;
		$num  =~ s/\D//g;
		
		# 小計
		my $kei = $price * $num;
		$all += $kei;

		# 単価計算
		$price = comma($price);
		$kei   = comma($kei);
		$cart_now .= qq|・$name<div class="ta-r eiji">$price x $num = $kei</div>\n|;
	}
	$tmpl =~ s/!cart-now!/$cart_now/g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	for ($head, $foot) {
		s/!total!/comma($all)/ge;
	}

	# 在庫機能
	my %zan;
	if ($cf{stock}) {
		open(IN,"$cf{datadir}/stock.dat") or error("open err: stock.dat");
		while (<IN>) {
			my ($code,$zan) = split(/<>/);
			$zan{$code} = $zan;
		}
		close(IN);
	} else {
		$loop =~ s|<!-- zai -->.+?<!-- /zai -->||s;
	}
	
	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 検索ヒット
	for ($begin .. $end) {
		chomp($log[$_]);
		my ($code,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,@ops) = split(/<>/,$log[$_]);
		
		# 税表記
		my $tax;
		if ($cf{tax_per} == 0) {
			$tax = $cf{tax_class}[0];
		} elsif (defined $red{$code}) {
			$tax = $cf{tax_class}[1] . ":軽減税率対象";
		} else {
			$tax = $cf{tax_class}[1];
		}
		
		# 文字置き換え
		my $tmp = $loop;
		$tmp =~ s/!item!/$item/g;
		$tmp =~ s/!price!/comma($price)/eg;
		$tmp =~ s/!code!/$code/g;
		$tmp =~ s/!memo!/$memo/g;
		$tmp =~ s/!top_back!/$cf{top_back}/g;
		$tmp =~ s/!tax!/$tax/g;

		# オプション
		my $opt;
		for my $i (0 .. $#{$cf{options}}) {
			next if ($ops[$i] eq '');
			
			my ($key,$nam) = split(/,/,$cf{options}[$i]);
			$opt .= qq|$nam ： <select name="$key">\n|;
			foreach my $op ( split(/\s+/,$ops[$i]) ) {
				$opt .= qq|<option value="$op">$op</option>\n|;
			}
			$opt .= qq|</select><br>\n|;
		}
		$tmp =~ s/<!-- option -->/$opt/g;
		
		# 在庫の場合
		if ($cf{stock} == 1) {
			$tmp =~ s/!zai!/defined($zan{$code}) ? $zan{$code} : 0/eg;
			if ($zan{$code} <= 0) {
				$tmp =~ s|<!-- submit -->.*<input ([^<>]+)>.*<!-- /submit -->|<input type="button" value="品切れ" disabled>|s;
			}
		}
		
		# 画像
		my ($ex,$w,$h) = split(/,/,$img1);
		if (-f "$cf{imgdir}/$code-1$ex") {

			$tmp =~ s|<!-- image -->|<img src="$cf{imgurl}/$code-1$ex" alt="$code" class="find-img">|;
		}
		
		print $tmp;
	}
	
	# フッター
	print footer($foot);
	exit;
}

#-----------------------------------------------------------
#  お知らせ画面
#-----------------------------------------------------------
sub read_news {
	
	# サイドデータ
	open(IN,"$cf{datadir}/news.dat") or error('open err: $cf{datadir}/news.dat');
	my $news = join('', <IN>);
	close(IN);

	# タグ/改行
	$news =~ s|\n|<br />|g;

	# 結果を返す
	return $news;
}

#-----------------------------------------------------------
#  ページボタン作成
#-----------------------------------------------------------
sub make_btn {
	my ($i,$pg) = @_;
	
	# 引数
	my $param = 'word=' . url_encode($in{word}) . "&amp;sort=$in{sort}";
	
	# ページ繰越数
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};
	
	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<span class="pg-on">$x</span>!);
			} else {
				push(@pg,qq!<span class="pg-off"><a href="$cf{find_cgi}?$param&amp;pg=$y">$x</a></span>!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<span class="pg-off"><a href="$cf{find_cgi}?$param&amp;pg=$back">&lt;</a></span>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<span class="pg-off"><a href="$cf{find_cgi}?$param&amp;pg=$next">&gt;</a></span>\n!;
	}
	$ret;
}

#-----------------------------------------------------------
#  URLエンコード
#-----------------------------------------------------------
sub url_encode {
	my $wd = shift;
	$wd =~ s/([^\w ])/'%'.unpack('H2', $1)/eg;
	$wd =~ tr/ /+/;
	return $wd;
}

#-----------------------------------------------------------
#  クッキー取得(cart)
#-----------------------------------------------------------
sub get_cookie_cart {
	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};
	
	# 該当IDを取り出す
	my %cook;
	for ( split(/;/,$cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}
	
	# URLデコード
	my (@cart,@cook);
	for ( split(/<>/,$cook{$cf{cookie_cart}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
		s/[&"'<>]//g;
		
		push(@cart,$_);
	}
	for ( split(/<>/,$cook{$cf{cookie_cust}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}
	return (\@cart,\@cook);
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	$ENV{HTTP_COOKIE} =~ /$cf{cookie_find}=([^=;]+);?/;
	my $back = $1;
	$back =~ s/\s//g;
	
	# URLデコード
	$back =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
	
	# URL書式をチェック
	if ($back =~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/i) {
		return $back;
	} else {
		return '/';
	}
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my $back = shift;
	
	if ($back =~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/i) {
		
		# URLエンコード
		my $url_enc = $back;
		$url_enc =~ s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		
		return ($back,$url_enc);
	}
}

