#!/usr/bin/perl

#┌─────────────────────────────────
#│ Web Shop v1.0 : index.cgi - 2022/11/17
#│ copyright (c) TRANSFORM,  Since 2012
#│ https://www.transformsite.com/
#└─────────────────────────────────

# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;
use CGI::Session;
use vars qw($cgi %in %cf);

# 外部ファイル取り込み
require './init.cgi';
%cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
%in = parse_form($cgi);

# 処理分岐
if ($in{mode_purchase})  { purchase_page(); }
if ($in{mode_paycheck})  { paycheck_page(); }
if ($in{mode_delivery})  { delivery_page(); }
if ($in{mode_policy})  { policy_page(); }
if ($in{mode_contact})  { contact_page(); }

shop_list();

#-----------------------------------------------------------
#  リスト一覧
#-----------------------------------------------------------
sub shop_list {
	# 詳細画面のとき
	detail_form() if ($in{mode} eq 'detail');

	# 引数
	$in{pg} =~ s/\D//g;
	
	# ページ数定義
	my $pg = $in{pg} || 0;
	
	# 軽減税率
	my %red = read_redtax() if ($cf{tax_per} > 0);

	# 在庫機能
	my %zan;
	if ($cf{stock}) {
		open(IN,"$cf{datadir}/stock.dat") or error("open err: stock.dat");
		while (<IN>) {
			my ($code,$zan) = split(/<>/);
			$zan{$code} = $zan;
		}
		close(IN);
	}
	
	# データファイルオープン
	my (@log,$i);
	open(IN,"$cf{datadir}/shop.dat") or error("open err: $cf{datadir}/shop.dat");
	while(<IN>) {
		my ($code,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,@opt) = split(/<>/);

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pg_max});

		push(@log,$_);
	}
	close(IN);

	# ヒット数
	my $hits = @log;
	
	# ページ繰越ボタン作成
	my $pgbtn = make_pgbtn($i,$pg);

	# 表示範囲
	#my $begin = $pg;
	#my $end   = $pg + $cf{pg_max} - 1;
	#if ($end > $hits - 1) { $end = $hits - 1; }

	# オプション
	my %sort = (1 => '価格の安い順', 2 => '価格の高い順', 3 => '登録の新着順', 4 => '登録の古い順');
	my $op_sort;
	foreach (1 .. 4) {
		if ($in{sort} == $_) {
			$op_sort .= qq|<option value="$_" selected>$sort{$_}</option>\n|;
		} else {
			$op_sort .= qq|<option value="$_">$sort{$_}</option>\n|;
		}
	}

	# サイドメニュー内お知らせ
	my $news = read_news();

	# 買物情報取得
	my ($cart,undef) = get_cookie_cart();

	# 商品データ認識
	my %cart = get_data();

	# 商品内容を展開
	my $cart_now;
	my $all = 0;
	for my $ck (@{$cart}) {
		my ($id,$code,$num,@op) = split(/,/,$ck);
			next if (!$cart);
		my (undef,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,@ops) = split(/<>/,$cart{$code});
		
		# チェック
		$id =~ s/\D//g;
		$code =~ s/\W//g;
		$num  =~ s/\D//g;
		
		# オプション処理
		my ($note,@op2);
		for my $i (0 .. $#{$cf{options}}) {
			my ($key,$nam) = split(/,/,$cf{options}[$i]);
			$op2[$i] = [split(/\s+/,$ops[$i])];
			
			if ($op[$i] ne '') { $note .= "[$nam] $op[$i]&nbsp; "; }
		}

		# 小計
		my $kei = $price * $num;
		$all += $kei;

		# 単価計算
		$price = comma($price);
		$kei   = comma($kei);
		$cart_now .= qq|・$item $note<div class="ta-r eiji">$price x $num = $kei</div>\n|;
	}
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/top.html") or error('open err: top.html');
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 置き換え
	$tmpl =~ s/!(\w+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/<!-- news -->/$news/;
	$tmpl =~ s/!cart_title!/$cf{cart_title}/g;
	$tmpl =~ s/!home!/$cf{home}/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s/!top_back!/$cf{top_back}/g;
	$tmpl =~ s|<!-- page_btn -->|<p>$pgbtn</p>|g;
	$tmpl =~ s/<!-- op_sort -->/$op_sort/g;
	$tmpl =~ s/!cart-now!/$cart_now/g;
	$tmpl =~ s/!pg!/$pg/g;
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");

	for ($head, $foot) {
		s/!total!/comma($all)/ge;
	}

	if (!$cf{stock}) {
		$loop =~ s|<!-- zai -->.+?<!-- /zai -->||s;
	}

	# クッキー取得
	my @cook = get_cookie();
	
	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 表示データ展開
	foreach (@log) {
		#chomp($log[$_]);
		my ($code,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,@ops) = split(/<>/);
		
		# 税表記
		my $tax;
		if ($cf{tax_per} == 0) {
			$tax = $cf{tax_class}[0];
		} elsif (defined $red{$code}) {
			$tax = $cf{tax_class}[1] . ":軽減税率対象";
		} else {
			$tax = $cf{tax_class}[1];
		}

		# プルダウン生成
		my $sel_num;
		for my $i (1 .. $cf{max_select}) {
			if ($cook[3] == $i) {
				$sel_num .= qq|<option value="$i" selected>$i</option>\n|;
			} else {
				$sel_num .= qq|<option value="$i">$i</option>\n|;
			}
		}

		$memo = cut_str($memo,$cf{memo_len});
		$item = cut_str($item,$cf{item_len});

		# 文字置き換え
		my $tmp = $loop;
		$tmp =~ s/!item!/$item/g;
		$tmp =~ s/!seq!/$code/g;
		$tmp =~ s/!price!/comma($price)/eg;
		$tmp =~ s/!code!/$code/g;
		$tmp =~ s/!memo!/$memo/g;
		$tmp =~ s/<!-- sel_num -->/$sel_num/g;
		$tmp =~ s/!top_back!/$cf{top_back}/g;
		$tmp =~ s/!tax!/$tax/g;
		$tmp =~ s|<!-- detail -->|<input type="button" id="detail" value="" onclick="window.open('$cf{index_cgi}?mode=detail&amp;no=$code','_self')" />|;

		# オプション
		my $opt;
		for my $i (0 .. $#{$cf{options}}) {
			next if ($ops[$i] eq '');
			
			my ($key,$nam) = split(/,/,$cf{options}[$i]);
			$opt .= qq|$nam ： <select name="$key">\n|;
			foreach my $op ( split(/\s+/,$ops[$i]) ) {
				$opt .= qq|<option value="$op">$op</option>\n|;
			}
			$opt .= qq|</select><br>\n|;
		}
		$tmp =~ s/<!-- option -->/$opt/g;

		# 在庫の場合
		if ($cf{stock} == 1) {
			$tmp =~ s/!zai!/defined($zan{$code}) ? $zan{$code} : 0/eg;
			if ($zan{$code} <= 0) {
				$tmp =~ s|<!-- submit -->.*<input ([^<>]+)>.*<!-- /submit -->|<input type="button" value="品切れ" disabled>|s;
			}
		} else {
			$tmp =~ s|<!-- stock -->.+?<!-- /stock -->||s;
		}

		# メイン画像
		my $image;
		if ($img1) {
			my ($ex,$w,$h) = split(/,/,$img1);
			$image = image(1,$code,$ex,$w,$h,$item);
		}

		my $param = "mode=detail&amp;no=$code";
		$param .= "&amp;pg=$pg" if ($pg);

		$tmp =~ s/<!-- image -->/$image/g;
		
		print $tmp;
	}

	# フッター
	print footer($foot);
	exit;
}

#-----------------------------------------------------------
#  文字カット for UTF-8
#-----------------------------------------------------------
sub cut_str {
	my ($str,$all) = @_;
	$str =~ s|<br( /)?>||g;
	
	my $i = 0;
	my ($ret,$flg);
	while ($str =~ /([\x00-\x7f]|[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/gx) {
		$i++;
		$ret .= $1;
		if ($i >= $all) {
			$flg++;
			last;
		}
	}
	$ret .= '...' if ($flg);
	
	return $ret;
}

#-----------------------------------------------------------
#  詳細画面
#-----------------------------------------------------------
sub detail_form {
	# 在庫機能
	my %zan;
	if ($cf{stock}) {
		open(IN,"$cf{datadir}/stock.dat") or error("open err: stock.dat");
		while (<IN>) {
			my ($code,$zan) = split(/<>/);
			$zan{$code} = $zan;
		}
		close(IN);
	}

	my $pg = $in{pg} || 0;
	$in{no} =~ s/\D//g;
	my ($log,$code);
	open(IN,"$cf{datadir}/shop.dat") or error("open err: $cf{datadir}/shop.dat");
	while (<IN>) {
		my ($code) = (split(/<>/))[0];

		if ($in{no} == $code) {

			$code = $code;

			$log = $_;
			last;
		}
	}
	close(IN);

	error('表示できません') if (!$log);

	# データ分解
	my ($code,$item,$price,$memo,$img1,$img2,$img3,$img4,$img5,@ops) = split(/<>/,$log);

	# テンプレート読込
	open(IN,"$cf{tmpldir}/detail.html") or error("open err: detail.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# オプション(検索ソートタイプ)
	my %sort = (1 => '価格の安い順', 2 => '価格の高い順', 3 => '登録の新着順', 4 => '登録の古い順');
	my $op_sort;
	foreach (1 .. 4) {
		if ($in{sort} == $_) {
			$op_sort .= qq|<option value="$_" selected>$sort{$_}</option>\n|;
		} else {
			$op_sort .= qq|<option value="$_">$sort{$_}</option>\n|;
		}
	}

		# オプション
		my $opt;
		for my $i (0 .. $#{$cf{options}}) {
			next if ($ops[$i] eq '');
			
			my ($key,$nam) = split(/,/,$cf{options}[$i]);
			$opt .= qq|$nam ： <select name="$key">\n|;
			foreach my $op ( split(/\s+/,$ops[$i]) ) {
				$opt .= qq|<option value="$op">$op</option>\n|;
			}
			$opt .= qq|</select><br>\n|;
		}
		$tmpl =~ s/<!-- option -->/$opt/g;

		# 在庫の場合
		if ($cf{stock} == 1) {
			$tmpl =~ s/!zai!/defined($zan{$code}) ? $zan{$code} : 0/eg;
			if ($zan{$code} <= 0) {
				$tmpl =~ s|<!-- submit -->.*<input ([^<>]+)>.*<!-- /submit -->|<input type="button" value="品切れ" disabled>|s;
			}
		} else {
			$tmpl =~ s|<!-- stock -->.+?<!-- /stock -->||s;
		}

	# 画像
	my ($i,$image);
	foreach my $img ($img1,$img2,$img3,$img4,$img5) {
		$i++;
		my ($ex,$w,$h) = split(/,/, $img);

		if ($ex) { $image .= image($i,$code,$ex,$w,$h,$item); }
	}
		$tmpl =~ s/<!-- image -->/$image/g;

	# サイドメニュー内お知らせ
	my $news = read_news();

	# 買物情報取得
	my ($cart,undef) = get_cookie_cart();

	# 商品データ認識
	my %cart = get_data();

	# 商品内容を展開
	my $cart_now;
	my $all = 0;
	for (@{$cart}) {
		my ($id,$code,$num,@op) = split(/,/);
			next if (!$cart);
		my (undef,$name,$price,undef,undef) = split(/<>/,$cart{$code});
		
		# チェック
		$id =~ s/\D//g;
		$code =~ s/\W//g;
		$num  =~ s/\D//g;
		
		# 小計
		my $kei = $price * $num;
		$all += $kei;

		# 単価計算
		$price = comma($price);
		$kei   = comma($kei);
		$cart_now .= qq|・$name<div class="ta-r eiji">$price x $num = $kei</div>\n|;
	}

	# 文字置き換え
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!cart_title!/$cf{cart_title}/g;
	$tmpl =~ s/!cart-now!/$cart_now/g;
	$tmpl =~ s/!total!/comma($all)/eg;
	$tmpl =~ s/!back!/$in{back}/g;
	$tmpl =~ s/!top_back!/$cf{top_back}/g;
	$tmpl =~ s/!homepage!/$cf{homepage}/g;
	$tmpl =~ s/<!-- news -->/$news/;
	$tmpl =~ s/!item-name!/$item/g;
	$tmpl =~ s/<!-- op_sort -->/$op_sort/g;
	$tmpl =~ s/!code!/$code/g;
	$tmpl =~ s/!price!/comma($price)/eg;
	$tmpl =~ s/!memo!/$memo/g;

	open(IN,"$cf{datadir}/detail/$code.dat");
	my $data = join('',<IN>);
	close(IN);
	
	$data =~ s/\n/<br>/g;
	$tmpl =~ s/!detail!/$data/g;

	# ヘッダ表示
	print "Content-type: text/html; charset=utf-8\n\n";
	footer($tmpl);
}

#-----------------------------------------------------------
#  お知らせ画面
#-----------------------------------------------------------
sub read_news {
	
	# サイドデータ
	open(IN,"$cf{datadir}/news.dat") or error('open err: $cf{datadir}/news.dat');
	my $news = join('', <IN>);
	close(IN);

	# タグ/改行
	$news =~ s|\n|<br />|g;

	# 結果を返す
	return $news;
}

#-----------------------------------------------------------
#  繰越ボタン作成
#-----------------------------------------------------------
sub make_pgbtn {
	my ($i,$pg) = @_;
	$cf{pg_max} ||= 5;

	# ページ繰越定義
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 or $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$cf{index_cgi}?pg=$y">$x</a></li>\n!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{index_cgi}?pg=$back">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{index_cgi}?pg=$next">&raquo;</a></li>\n!;
	}
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  検索ページボタン作成
#-----------------------------------------------------------
sub make_btn {
	my ($i,$pg) = @_;
	
	# 引数
	my $param = 'word=' . url_encode($in{word}) . "&amp;sort=$in{sort}";
	
	# ページ繰越数
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};
	
	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<span class="pg-on">$x</span>!);
			} else {
				push(@pg,qq!<span class="pg-off"><a href="$cf{find_cgi}?$param&amp;pg=$y">$x</a></span>!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<span class="pg-off"><a href="$cf{find_cgi}?$param&amp;pg=$back">&lt;</a></span>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<span class="pg-off"><a href="$cf{find_cgi}?$param&amp;pg=$next">&gt;</a></span>\n!;
	}
	$ret;
}

#-----------------------------------------------------------
#  URLエンコード
#-----------------------------------------------------------
sub url_encode {
	my $wd = shift;
	$wd =~ s/([^\w ])/'%'.unpack('H2', $1)/eg;
	$wd =~ tr/ /+/;
	return $wd;
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	$ENV{HTTP_COOKIE} =~ /$cf{cookie_find}=([^=;]+);?/;
	my $back = $1;
	$back =~ s/\s//g;
	
	# URLデコード
	$back =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
	
	# URL書式をチェック
	if ($back =~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/i) {
		return $back;
	} else {
		return '/';
	}
}

#-----------------------------------------------------------
#  クッキー取得(cart)
#-----------------------------------------------------------
sub get_cookie_cart {
	# クッキー取得
	my $cook = $ENV{HTTP_COOKIE};
	
	# 該当IDを取り出す
	my %cook;
	for ( split(/;/,$cook) ) {
		my ($key,$val) = split(/=/);
		$key =~ s/\s//g;
		$cook{$key} = $val;
	}
	
	# URLデコード
	my (@cart,@cook);
	for ( split(/<>/,$cook{$cf{cookie_cart}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
		s/[&"'<>]//g;
		
		push(@cart,$_);
	}
	for ( split(/<>/,$cook{$cf{cookie_cust}}) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}
	return (\@cart,\@cook);
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my $back = shift;
	
	if ($back =~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/i) {
		
		# URLエンコード
		my $url_enc = $back;
		$url_enc =~ s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		
		return ($back,$url_enc);
	}
}
#-----------------------------------------------------------
#  画像表示
#-----------------------------------------------------------
sub image {
	my ($i,$no,$ex,$w,$h,$item) = @_;

	my $image;
	if (-f "$cf{imgdir}/$no-s-$i$ex") {
		$image = qq|<img src="$cf{imgurl}/$no-s-$i$ex" alt="" />|;

	} elsif ($in{mode} eq 'detail') {
		($w,$h) = resize_detail($w,$h);
		$image = qq|<img src="$cf{imgurl}/$no-$i$ex" width="$w" height="$h" alt="" />|;
	} else {
		($w,$h) = resize($w,$h);
		$image = qq|<img src="$cf{imgurl}/$no-$i$ex" width="$w" height="$h" alt="" class="list-img" />|;
	}

	my $pg = $in{pg} || 0;
		my $param = "mode=detail&amp;no=$no";
		$param .= "&amp;pg=$pg" if ($pg);

	if ($in{mode} eq 'detail') {
		return qq|<a href="$cf{imgurl}/$no-$i$ex" title="商品名：$item" data-lighter>$image</a>|;
	} else {
		return qq|<a href="$cf{cart_cgi}?$param">$image</a>|;
	}
}

#-----------------------------------------------------------
#  お買い物方法の表記
#-----------------------------------------------------------
sub purchase_page {
	require './lib/purchase.pl';
	purchase_pl();
}

#-----------------------------------------------------------
#  お支払い方法の表記
#-----------------------------------------------------------
sub paycheck_page {
	require './lib/paycheck.pl';
	paycheck_pl();
}

#-----------------------------------------------------------
#  配送方法・送料の表記
#-----------------------------------------------------------
sub delivery_page {
	require './lib/delivery.pl';
	delivery_pl();
}

#-----------------------------------------------------------
#  プライバシー・ポリシーの表記
#-----------------------------------------------------------
sub policy_page {
	require './lib/policy.pl';
	policy_pl();
}

#-----------------------------------------------------------
#  お問合せ
#-----------------------------------------------------------
sub contact_page {
	require './lib/contact.pl';
	contact_pl();
}

